@@ -11,8 +11,8 @@ import shutil
 import tempfile
 from textwrap import dedent
 
+import cloudpickle
 from flaky import flaky
-from ipykernel.serialize import serialize_object
 from pygments.token import Name
 import pytest
 from qtpy import PYQT4, PYQT5, PYQT_VERSION
@@ -26,6 +26,7 @@ from spyder.plugins.ipythonconsole import (IPythonConsole,
                                            KernelConnectionDialog)
 from spyder.utils.environ import listdict2envdict
 from spyder.utils.ipython.style import create_style_class
+from spyder.utils.programs import TEMPDIR
 from spyder.utils.test import close_message_box
 from spyder.widgets.variableexplorer.collectionseditor import CollectionsEditor
 
@@ -36,6 +37,7 @@ from spyder.widgets.variableexplorer.collectionseditor import CollectionsEditor
 SHELL_TIMEOUT = 20000
 PYQT_WHEEL = PYQT_VERSION > '5.6'
 TEMP_DIRECTORY = tempfile.gettempdir()
+NON_ASCII_DIR = osp.join(TEMP_DIRECTORY, u'測試', u'اختبار')
 
 
 #==============================================================================
@@ -64,41 +66,91 @@ def get_console_background_color(style_sheet):
 #==============================================================================
 @pytest.fixture
 def ipyconsole(request):
-    try:
-        console = IPythonConsole(None, testing=True, test_dir=request.param)
-    except AttributeError:
-        console = IPythonConsole(None, testing=True)
+    """IPython console fixture."""
+
+    # Test the console with a non-ascii temp dir
+    non_ascii_dir = request.node.get_marker('non_ascii_dir')
+    if non_ascii_dir:
+        test_dir = NON_ASCII_DIR
+    else:
+        test_dir = TEMPDIR
+
+    # Instruct the console to not use a stderr file
+    no_stderr_file = request.node.get_marker('no_stderr_file')
+    if no_stderr_file:
+        test_no_stderr = True
+    else:
+        test_no_stderr = False
+
+    # Create the console and a new client
+    console = IPythonConsole(parent=None,
+                             testing=True,
+                             test_dir=test_dir,
+                             test_no_stderr=test_no_stderr)
     console.create_new_client()
+
     def close_console():
         console.closing_plugin()
         console.close()
     request.addfinalizer(close_console)
     console.show()
+
     return console
 
 
 #==============================================================================
 # Tests
 #==============================================================================
-@pytest.mark.parametrize('ipyconsole', [osp.join(TEMP_DIRECTORY, u'測試',
-                                                 u'اختبار')], indirect=True)
+@flaky(max_runs=3)
 @pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
-def test_console_stderr_file(ipyconsole, qtbot):
-    """Test a the creation of a console with a stderr file in ascii dir."""
+def test_conf_env_vars(ipyconsole, qtbot):
+    """Test that kernels have env vars set by our kernel spec."""
     # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
 
-    # Create a new client with s stderr file in a non-ascii dir
-    ipyconsole.create_new_client()
+    # Get a CONF env var
+    with qtbot.waitSignal(shell.executed):
+        shell.execute("import os; a = os.environ.get('SPY_SYMPY_O')")
+
+    # Assert we get the assigned value correctly
+    assert shell.get_value('a') == 'False'
+
+
+@flaky(max_runs=3)
+@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
+@pytest.mark.no_stderr_file
+def test_no_stderr_file(ipyconsole, qtbot):
+    """Test that consoles can run without an stderr."""
+    # Wait until the window is fully up
     shell = ipyconsole.get_current_shellwidget()
     qtbot.waitUntil(lambda: shell._prompt_html is not None,
                     timeout=SHELL_TIMEOUT)
+
+    # Execute a simple assignment
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 1')
 
-    # Assert we get the a value correctly
+    # Assert we get the assigned value correctly
+    assert shell.get_value('a') == 1
+
+
+@flaky(max_runs=3)
+@pytest.mark.skipif(os.name == 'nt', reason="It times out sometimes on Windows")
+@pytest.mark.non_ascii_dir
+def test_non_ascii_stderr_file(ipyconsole, qtbot):
+    """Test the creation of a console with a stderr file in a non-ascii dir."""
+    # Wait until the window is fully up
+    shell = ipyconsole.get_current_shellwidget()
+    qtbot.waitUntil(lambda: shell._prompt_html is not None,
+                    timeout=SHELL_TIMEOUT)
+
+    # Execute a simple assignment
+    with qtbot.waitSignal(shell.executed):
+        shell.execute('a = 1')
+
+    # Assert we get the assigned value
     assert shell.get_value('a') == 1
 
 
@@ -288,7 +340,7 @@ def test_unicode_vars(ipyconsole, qtbot):
     assert shell.get_value('д') == 10
 
     # Change its value and verify
-    shell.set_value('д', serialize_object(20))
+    shell.set_value('д', [cloudpickle.dumps(20, protocol=2)])
     qtbot.wait(1000)
     assert shell.get_value('д') == 20
 
@@ -338,7 +390,7 @@ def test_values_dbg(ipyconsole, qtbot):
     assert shell.get_value('aa') == 10
 
     # Set value
-    shell.set_value('aa', serialize_object(20))
+    shell.set_value('aa', [cloudpickle.dumps(20, protocol=2)])
     qtbot.wait(1000)
     assert shell.get_value('aa') == 20
 
@@ -423,7 +475,7 @@ def test_run_doctest(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or (PY2 and PYQT5),
+@pytest.mark.skipif(os.name == 'nt' or (PY2 and PYQT5) or PYQT4,
                     reason="It times out frequently")
 def test_mpl_backend_change(ipyconsole, qtbot):
     """
@@ -454,7 +506,8 @@ def test_mpl_backend_change(ipyconsole, qtbot):
 
 
 @flaky(max_runs=10)
-@pytest.mark.skipif(PYQT5, reason="It times out frequently in PyQt5")
+@pytest.mark.skipif(os.environ.get('CI', None) is not None or PYQT5,
+                    reason="It fails frequently in PyQt5 and our CIs")
 def test_ctrl_c_dbg(ipyconsole, qtbot):
     """
     Test that Ctrl+C works while debugging
@@ -510,7 +563,7 @@ def test_clear_and_reset_magics_dbg(ipyconsole, qtbot):
     qtbot.wait(500)
     assert shell.get_value('bb') == 10
 
-    shell.reset_namespace(force=True)
+    shell.reset_namespace(warning=False, silent=True)
     qtbot.wait(1000)
 
     qtbot.keyClicks(control, '!bb')
@@ -530,7 +583,7 @@ def test_restart_kernel(ipyconsole, qtbot):
     client = ipyconsole.get_current_client()
     qtbot.waitUntil(lambda: shell._prompt_html is not None, timeout=SHELL_TIMEOUT)
 
-    # Do an assigment to verify that it's not there after restarting
+    # Do an assignment to verify that it's not there after restarting
     with qtbot.waitSignal(shell.executed):
         shell.execute('a = 10')
 
@@ -566,7 +619,8 @@ def test_load_kernel_file_from_id(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt', reason="It times out on Windows")
+@pytest.mark.skipif(os.name == 'nt' or (PY3 and PYQT4),
+                    reason="It segfaults frequently")
 def test_load_kernel_file_from_location(ipyconsole, qtbot):
     """
     Test that a new client is created using a connection file
@@ -590,7 +644,7 @@ def test_load_kernel_file_from_location(ipyconsole, qtbot):
 
 
 @flaky(max_runs=3)
-@pytest.mark.skipif(os.name == 'nt' or (PY3 and PYQT4),
+@pytest.mark.skipif(os.name == 'nt' or PYQT4,
                     reason="It segfaults frequently")
 def test_load_kernel_file(ipyconsole, qtbot):
     """
