@@ -11,50 +11,26 @@ Spyder kernel for Jupyter
 # Standard library imports
 import os
 import os.path as osp
+import sys
 
 # Third-party imports
-from ipykernel.datapub import publish_data
 from ipykernel.ipkernel import IPythonKernel
-import ipykernel.pickleutil
-from ipykernel.pickleutil import CannedObject
-from ipykernel.serialize import deserialize_object
+
+
+PY2 = sys.version[0] == '2'
 
 # Check if we are running under an external interpreter
-IS_EXT_INTERPRETER = os.environ.get('EXTERNAL_INTERPRETER', '').lower() == "true"
-
-# Local imports
-if not IS_EXT_INTERPRETER:
-    from spyder.py3compat import is_text_string
-    from spyder.utils.dochelpers import isdefined, getdoc, getsource
-    from spyder.utils.iofuncs import iofunctions
-    from spyder.utils.misc import fix_reference_name
-    from spyder.widgets.variableexplorer.utils import (get_remote_data,
-                                                       make_remote_view)
-else:
-    # We add "spyder" to sys.path for external interpreters, so this works!
-    # See create_kernel_spec of plugins/ipythonconsole
-    from py3compat import is_text_string
-    from utils.dochelpers import isdefined, getdoc, getsource
-    from utils.iofuncs import iofunctions
-    from utils.misc import fix_reference_name
-    from widgets.variableexplorer.utils import (get_remote_data,
-                                                make_remote_view)
-
-
-# XXX --- Disable canning for Numpy arrays for now ---
-# This allows getting values between a Python 3 frontend
-# and a Python 2 kernel, and viceversa, for several types of
-# arrays.
-# See this link for interesting ideas on how to solve this
-# in the future:
-# http://stackoverflow.com/q/30698004/438386
-ipykernel.pickleutil.can_map.pop('numpy.ndarray')
-
+# We add "spyder" to sys.path for external interpreters,
+# so relative imports work!
+IS_EXT_INTERPRETER = os.environ.get('SPY_EXTERNAL_INTERPRETER') == "True"
 
 # Excluded variables from the Variable Explorer (i.e. they are not
 # shown at all there)
 EXCLUDED_NAMES = ['In', 'Out', 'exit', 'get_ipython', 'quit']
 
+# To be able to get and set variables between Python 2 and 3
+PICKLE_PROTOCOL = 2
+
 
 class SpyderKernel(IPythonKernel):
     """Spyder kernel for Jupyter"""
@@ -63,8 +39,11 @@ class SpyderKernel(IPythonKernel):
         super(SpyderKernel, self).__init__(*args, **kwargs)
 
         self.namespace_view_settings = {}
+
         self._pdb_obj = None
         self._pdb_step = None
+        self._do_publish_pdb_state = True
+        self._mpl_backend_error = None
 
         kernel_config = self.config.get('IPKernelApp', None)
         if kernel_config is not None:
@@ -107,6 +86,11 @@ class SpyderKernel(IPythonKernel):
         * 'size' and 'type' are self-evident
         * and'view' is its value or the text shown in the last column
         """
+        if not IS_EXT_INTERPRETER:
+            from spyder.widgets.variableexplorer.utils import make_remote_view
+        else:
+            from widgets.variableexplorer.utils import make_remote_view
+
         settings = self.namespace_view_settings
         if settings:
             ns = self._get_current_namespace()
@@ -120,6 +104,11 @@ class SpyderKernel(IPythonKernel):
         Get some properties of the variables in the current
         namespace
         """
+        if not IS_EXT_INTERPRETER:
+            from spyder.widgets.variableexplorer.utils import get_remote_data
+        else:
+            from widgets.variableexplorer.utils import get_remote_data
+
         settings = self.namespace_view_settings
         if settings:
             ns = self._get_current_namespace()
@@ -144,27 +133,65 @@ class SpyderKernel(IPythonKernel):
         else:
             return repr(None)
 
+    def send_spyder_msg(self, spyder_msg_type, content=None, data=None):
+        """publish custom messages to the spyder frontend
+
+        Parameters
+        ----------
+
+        spyder_msg_type: str
+            The spyder message type
+        content: dict
+            The (JSONable) content of the message
+        data: any
+            Any object that is serializable by cloudpickle (should be most
+            things). Will arrive as cloudpickled bytes in `.buffers[0]`.
+        """
+        import cloudpickle
+
+        if content is None:
+            content = {}
+        content['spyder_msg_type'] = spyder_msg_type
+        self.session.send(
+            self.iopub_socket,
+            'spyder_msg',
+            content=content,
+            buffers=[cloudpickle.dumps(data, protocol=PICKLE_PROTOCOL)],
+            parent=self._parent_header,
+        )
+
     def get_value(self, name):
         """Get the value of a variable"""
         ns = self._get_current_namespace()
         value = ns[name]
         try:
-            publish_data({'__spy_data__': value})
+            self.send_spyder_msg('data', data=value)
         except:
             # * There is no need to inform users about
             #   these errors.
             # * value = None makes Spyder to ignore
             #   petitions to display a value
-            value = None
-            publish_data({'__spy_data__': value})
+            self.send_spyder_msg('data', data=None)
+        self._do_publish_pdb_state = False
 
-    def set_value(self, name, value):
+    def set_value(self, name, value, PY2_frontend):
         """Set the value of a variable"""
+        import cloudpickle
         ns = self._get_reference_namespace(name)
-        value = deserialize_object(value)[0]
-        if isinstance(value, CannedObject):
-            value = value.get_object()
-        ns[name] = value
+
+        # We send serialized values in a list of one element
+        # from Spyder to the kernel, to be able to send them
+        # at all in Python 2
+        svalue = value[0]
+
+        # We need to convert svalue to bytes if the frontend
+        # runs in Python 2 and the kernel runs in Python 3
+        if PY2_frontend and not PY2:
+            svalue = bytes(svalue, 'latin-1')
+
+        # Deserialize and set value in namespace
+        dvalue = cloudpickle.loads(svalue)
+        ns[name] = dvalue
 
     def remove_value(self, name):
         """Remove a variable"""
@@ -178,6 +205,13 @@ class SpyderKernel(IPythonKernel):
 
     def load_data(self, filename, ext):
         """Load data from filename"""
+        if not IS_EXT_INTERPRETER:
+            from spyder.utils.iofuncs import iofunctions
+            from spyder.utils.misc import fix_reference_name
+        else:
+            from utils.iofuncs import iofunctions
+            from utils.misc import fix_reference_name
+
         glbs = self._mglobals()
 
         load_func = iofunctions.load_funcs[ext]
@@ -200,6 +234,13 @@ class SpyderKernel(IPythonKernel):
 
     def save_namespace(self, filename):
         """Save namespace into filename"""
+        if not IS_EXT_INTERPRETER:
+            from spyder.utils.iofuncs import iofunctions
+            from spyder.widgets.variableexplorer.utils import get_remote_data
+        else:
+            from utils.iofuncs import iofunctions
+            from widgets.variableexplorer.utils import get_remote_data
+
         ns = self._get_current_namespace()
         settings = self.namespace_view_settings
         data = get_remote_data(ns, settings, mode='picklable',
@@ -207,35 +248,63 @@ class SpyderKernel(IPythonKernel):
         return iofunctions.save(data, filename)
 
     # --- For Pdb
-    def get_pdb_step(self):
-        """Return info about pdb current frame"""
-        return self._pdb_step
-
     def publish_pdb_state(self):
         """
         Publish Variable Explorer state and Pdb step through
-        publish_data.
+        send_spyder_msg.
         """
-        if self._pdb_obj:
+        if self._pdb_obj and self._do_publish_pdb_state:
             state = dict(namespace_view = self.get_namespace_view(),
                          var_properties = self.get_var_properties(),
                          step = self._pdb_step)
-            publish_data({'__spy_pdb_state__': state})
+            self.send_spyder_msg('pdb_state', content={'pdb_state': state})
+        self._do_publish_pdb_state = True
+
+    def pdb_continue(self):
+        """
+        Tell the console to run 'continue' after entering a
+        Pdb session to get to the first breakpoint.
+
+        Fixes issue 2034
+        """
+        if self._pdb_obj:
+            self.send_spyder_msg('pdb_continue')
 
     # --- For the Help plugin
     def is_defined(self, obj, force_import=False):
         """Return True if object is defined in current namespace"""
+        if not IS_EXT_INTERPRETER:
+            from spyder.utils.dochelpers import isdefined
+        else:
+            from utils.dochelpers import isdefined
+
         ns = self._get_current_namespace(with_magics=True)
         return isdefined(obj, force_import=force_import, namespace=ns)
 
     def get_doc(self, objtxt):
         """Get object documentation dictionary"""
+        try:
+            import matplotlib
+            matplotlib.rcParams['docstring.hardcopy'] = True
+        except:
+            pass
+
+        if not IS_EXT_INTERPRETER:
+            from spyder.utils.dochelpers import getdoc
+        else:
+            from utils.dochelpers import getdoc
+
         obj, valid = self._eval(objtxt)
         if valid:
             return getdoc(obj)
 
     def get_source(self, objtxt):
         """Get object source"""
+        if not IS_EXT_INTERPRETER:
+            from spyder.utils.dochelpers import getsource
+        else:
+            from utils.dochelpers import getsource
+
         obj, valid = self._eval(objtxt)
         if valid:
             return getsource(obj)
@@ -251,7 +320,6 @@ class SpyderKernel(IPythonKernel):
 
     def get_syspath(self):
         """Return sys.path contents."""
-        import sys
         return sys.path[:]
 
     def get_env(self):
@@ -395,9 +463,68 @@ class SpyderKernel(IPythonKernel):
         where *obj* is the object represented by *text*
         and *valid* is True if object evaluation did not raise any exception
         """
+        if not IS_EXT_INTERPRETER:
+            from spyder.py3compat import is_text_string
+        else:
+            from py3compat import is_text_string
+
         assert is_text_string(text)
         ns = self._get_current_namespace(with_magics=True)
         try:
             return eval(text, ns), True
         except:
             return None, False
+
+    # --- For Matplotlib
+    def _set_mpl_backend(self, backend, pylab=False):
+        """
+        Set a backend for Matplotlib.
+
+        backend: A parameter that can be passed to %matplotlib
+                 (e.g. inline or tk).
+        """
+        import traceback
+        from IPython.core.getipython import get_ipython
+
+        generic_error = ("\n"
+                         "NOTE: The following error appeared when setting "
+                         "your Matplotlib backend\n\n"
+                         "{0}")
+
+        error = None
+        try:
+            if pylab:
+                get_ipython().run_line_magic('pylab', backend)
+            else:
+                get_ipython().run_line_magic('matplotlib', backend)
+        except RuntimeError as err:
+            # This catches errors generated by ipykernel when
+            # trying to set a backend. See issue 5541
+            if "GUI eventloops" in str(err):
+                import matplotlib
+                previous_backend = matplotlib.get_backend()
+                error = ("\n"
+                         "NOTE: Spyder *can't* set your selected Matplotlib "
+                         "backend because there is a previous backend already "
+                         "in use.\n\n"
+                         "Your backend will be {0}".format(previous_backend))
+                del matplotlib
+            # This covers other RuntimeError's
+            else:
+                error = generic_error.format(traceback.format_exc())
+        except Exception:
+            error = generic_error.format(traceback.format_exc())
+
+        self._mpl_backend_error = error
+
+    def _show_mpl_backend_errors(self):
+        """Show Matplotlib backend errors after the prompt is ready."""
+        if self._mpl_backend_error is not None:
+            print(self._mpl_backend_error)  # spyder: test-skip
+
+    # --- Others
+    def _load_autoreload_magic(self):
+        """Load %autoreload magic."""
+        from IPython.core.getipython import get_ipython
+        get_ipython().run_line_magic('reload_ext', 'autoreload')
+        get_ipython().run_line_magic('autoreload', '2')
