@@ -21,7 +21,7 @@ from qtpy.QtGui import QTextCursor
 
 # Local imports
 from spyder.utils.fixtures import setup_editor
-from spyder.widgets.editor import EditorStack
+from spyder.widgets.editor import EditorStack, EditorSplitter
 from spyder.widgets.findreplace import FindReplace
 from spyder.py3compat import PY2
 
@@ -82,6 +82,16 @@ def editor_cells_bot(base_editor_bot):
     return editor_stack, finfo.editor, qtbot
 
 
+@pytest.fixture
+def editor_splitter_bot(qtbot):
+    """Create editor splitter."""
+    editor_splitter = EditorSplitter(None, Mock(), [], first=True)
+    qtbot.addWidget(editor_splitter)
+    editor_splitter.show()
+    yield editor_splitter
+    editor_splitter.destroy()
+
+
 # Tests
 #-------------------------------
 def test_find_number_matches(qtbot):
@@ -310,22 +320,60 @@ def test_replace_enter_press(editor_find_replace_bot):
     qtbot.keyPress(finder.search_text, Qt.Key_Return, modifier=Qt.ShiftModifier)
     assert editor.get_cursor_line_column() == (3,4)
 
-
-def test_selection_replace_plain_regex(editor_find_replace_bot):
+def test_replace_plain_regex(editor_find_replace_bot):
     """Test that regex reserved characters are displayed as plain text."""
     editor_stack, editor, finder, qtbot = editor_find_replace_bot
     expected_new_text = ('.\\[()]*test bacon\n'
                          'spam sausage\n'
                          'spam egg')
-    old_text = editor.toPlainText()
     finder.show()
     finder.show_replace()
     qtbot.keyClicks(finder.search_text, 'spam')
     qtbot.keyClicks(finder.replace_text, '.\[()]*test')
     qtbot.keyPress(finder.replace_text, Qt.Key_Return)
-    text = editor.toPlainText()[0:-1]
     assert editor.toPlainText()[0:-1] == expected_new_text
 
+def test_replace_invalid_regex(editor_find_replace_bot):
+    """Assert that replacing an invalid regexp does nothing."""
+    editor_stack, editor, finder, qtbot = editor_find_replace_bot
+    old_text = editor.toPlainText()
+    finder.show()
+    finder.show_replace()
+    qtbot.keyClicks(finder.search_text, '\\')
+    qtbot.keyClicks(finder.replace_text, 'anything')
+    if not finder.re_button.isChecked():
+        qtbot.mouseClick(finder.re_button, Qt.LeftButton)
+    qtbot.mouseClick(finder.replace_button, Qt.LeftButton)
+    assert editor.toPlainText() == old_text
+    qtbot.mouseClick(finder.replace_sel_button, Qt.LeftButton)
+    assert editor.toPlainText() == old_text
+    qtbot.mouseClick(finder.replace_all_button, Qt.LeftButton)
+    assert editor.toPlainText() == old_text
+
+
+def test_selection_escape_characters(editor_find_replace_bot):
+    editor_stack, editor, finder, qtbot = editor_find_replace_bot
+    expected_new_text = ('spam bacon\n'
+                         'spam sausage\n'
+                         'spam egg\n'
+                         '\\n \\t some escape characters')
+    qtbot.keyClicks(editor, '\\n \\t escape characters')
+
+    finder.show()
+    finder.show_replace()
+    qtbot.keyClicks(finder.search_text, 'escape')
+    qtbot.keyClicks(finder.replace_text, 'some escape')
+
+    # Select last line
+    cursor = editor.textCursor()
+    cursor.select(QTextCursor.LineUnderCursor)
+    assert cursor.selection().toPlainText() == "\\n \\t escape characters"
+
+    #replace
+    finder.replace_find_selection()
+    assert editor.toPlainText() == expected_new_text
+
+
 def test_advance_cell(editor_cells_bot):
     editor_stack, editor, qtbot = editor_cells_bot
 
@@ -395,5 +443,30 @@ def test_get_current_word(base_editor_bot):
     assert editor.get_current_word() == 'valid_python_word'
 
 
+def test_editor_splitter_init(editor_splitter_bot):
+    """"Test EditorSplitter.__init__."""
+    es = editor_splitter_bot
+    assert es.orientation() == Qt.Horizontal
+    assert es.testAttribute(Qt.WA_DeleteOnClose)
+    assert not es.childrenCollapsible()
+    assert not es.toolbar_list
+    assert not es.menu_list
+    assert es.register_editorstack_cb == es.plugin.register_editorstack
+    assert es.unregister_editorstack_cb == es.plugin.unregister_editorstack
+
+    # No menu actions in parameter call.
+    assert not es.menu_actions
+    # EditorStack adds its own menu actions to the existing actions.
+    assert es.editorstack.menu_actions
+
+    assert isinstance(es.editorstack, EditorStack)
+    es.plugin.register_editorstack.assert_called_with(es.editorstack)
+    es.plugin.unregister_editorstack.assert_not_called()
+    es.plugin.clone_editorstack.assert_not_called()
+
+    assert es.count() == 1
+    assert es.widget(0) == es.editorstack
+
+
 if __name__ == "__main__":
     pytest.main()
