@@ -5,18 +5,19 @@
 # (see spyder/__init__.py for details)
 
 """
-Utilities for the Collections editor widget and dialog
+Utilities
 """
 
 from __future__ import print_function
 
+from itertools import islice
 import re
 
 # Local imports
 from spyder.config.base import get_supported_types
 from spyder.py3compat import (NUMERIC_TYPES, TEXT_TYPES, to_text_string,
-                              is_text_string, is_binary_string, reprlib,
-                              PY2, to_binary_string)
+                              is_text_string, is_binary_string, PY2,
+                              to_binary_string, iteritems)
 from spyder.utils import programs
 from spyder import dependencies
 from spyder.config.base import _
@@ -48,15 +49,14 @@ class FakeObject(object):
 #==============================================================================
 try:
     from numpy import (ndarray, array, matrix, recarray,
-                      int64, int32, float64, float32,
-                      complex64, complex128)
+                       int64, int32, float64, float32,
+                       complex64, complex128)
     from numpy.ma import MaskedArray
     from numpy import savetxt as np_savetxt
-    from numpy import set_printoptions as np_set_printoptions
-except ImportError:
+    from numpy import get_printoptions, set_printoptions
+except:
     ndarray = array = matrix = recarray = MaskedArray = np_savetxt = \
-    np_set_printoptions = int64 = int32 = float64 = float32 = \
-    complex64 = complex128 = FakeObject
+    int64 = int32 = float64 = float32 = complex64 = complex128 = FakeObject
 
 def get_numpy_dtype(obj):
     """Return NumPy data type associated to obj
@@ -82,7 +82,10 @@ def get_numpy_dtype(obj):
 # Pandas support
 #==============================================================================
 if programs.is_module_installed('pandas', PANDAS_REQVER):
-    from pandas import DataFrame, DatetimeIndex, Series
+    try:
+        from pandas import DataFrame, DatetimeIndex, Series
+    except:
+        DataFrame = DatetimeIndex = Series = FakeObject
 else:
     DataFrame = DatetimeIndex = Series = FakeObject      # analysis:ignore
 
@@ -93,7 +96,7 @@ else:
 try:
     from spyder import pil_patch
     Image = pil_patch.Image.Image
-except ImportError:
+except:
     Image = FakeObject  # analysis:ignore
 
 
@@ -103,7 +106,7 @@ except ImportError:
 try:
     import bs4
     NavigableString = bs4.element.NavigableString
-except (ImportError, AttributeError):
+except:
     NavigableString = FakeObject  # analysis:ignore
 
 
@@ -150,20 +153,6 @@ def get_object_attrs(obj):
     return attrs
 
 
-# =============================================================================
-# Set limits for the amount of elements in the repr of collections (lists,
-# dicts, tuples and sets) and Numpy arrays
-# =============================================================================
-CollectionsRepr = reprlib.Repr()
-CollectionsRepr.maxlist = 10
-CollectionsRepr.maxdict = 10
-CollectionsRepr.maxtuple = 10
-CollectionsRepr.maxset = 10
-
-if np_set_printoptions is not FakeObject:
-    np_set_printoptions(threshold=10)
-
-
 #==============================================================================
 # Date and datetime objects support
 #==============================================================================
@@ -172,7 +161,7 @@ import datetime
 
 try:
     from dateutil.parser import parse as dateparse
-except ImportError:
+except:
     def dateparse(datestr):  # analysis:ignore
         """Just for 'year, month, day' strings"""
         return datetime.datetime( *list(map(int, datestr.split(','))) )
@@ -181,10 +170,38 @@ except ImportError:
 def datestr_to_datetime(value):
     rp = value.rfind('(')+1
     v = dateparse(value[rp:-1])
-    print(value, "-->", v)
+    print(value, "-->", v)  # spyder: test-skip
     return v
 
 
+def str_to_timedelta(value):
+    """Convert a string to a datetime.timedelta value.
+
+    The following strings are accepted:
+
+        - 'datetime.timedelta(1, 5, 12345)'
+        - 'timedelta(1, 5, 12345)'
+        - '(1, 5, 12345)'
+        - '1, 5, 12345'
+        - '1'
+
+    if there are less then three parameters, the missing parameters are
+    assumed to be 0. Variations in the spacing of the parameters are allowed.
+
+    Raises:
+        ValueError for strings not matching the above criterion.
+
+    """
+    m = re.match('^(?:(?:datetime\.)?timedelta)?'
+                 '\(?'
+                 '([^)]*)'
+                 '\)?$', value)
+    if not m:
+        raise ValueError('Invalid string for datetime.timedelta')
+    args = [int(a.strip()) for a in m.group(1).split(',')]
+    return datetime.timedelta(*args)
+
+
 #==============================================================================
 # Background colors for supported types
 #==============================================================================
@@ -205,6 +222,7 @@ COLORS = {
            DatetimeIndex):    ARRAY_COLOR,
           Image:              "#008000",
           datetime.date:      "#808000",
+          datetime.timedelta: "#808000",
           }
 CUSTOM_TYPE_COLOR = "#7755aa"
 UNSUPPORTED_COLOR = "#ffffff"
@@ -257,79 +275,177 @@ def unsorted_unique(lista):
 #==============================================================================
 # Display <--> Value
 #==============================================================================
-def value_to_display(value, minmax=False):
+def default_display(value, with_module=True):
+    """Default display for unknown objects."""
+    object_type = type(value)
+    try:
+        name = object_type.__name__
+        module = object_type.__module__
+        if with_module:
+            return name + ' object of ' + module + ' module'
+        else:
+            return name
+    except:
+        type_str = to_text_string(object_type)
+        return type_str[1:-1]
+
+
+def collections_display(value, level):
+    """Display for collections (i.e. list, tuple and dict)."""
+    is_dict = isinstance(value, dict)
+
+    # Get elements
+    if is_dict:
+        elements = iteritems(value)
+    else:
+        elements = value
+
+    # Truncate values
+    truncate = False
+    if level == 1 and len(value) > 10:
+        elements = islice(elements, 10) if is_dict else value[:10]
+        truncate = True
+    elif level == 2 and len(value) > 5:
+        elements = islice(elements, 5) if is_dict else value[:5]
+        truncate = True
+
+    # Get display of each element
+    if level <= 2:
+        if is_dict:
+            displays = [value_to_display(k, level=level) + ':' +
+                        value_to_display(v, level=level)
+                        for (k, v) in list(elements)]
+        else:
+            displays = [value_to_display(e, level=level)
+                        for e in elements]
+        if truncate:
+            displays.append('...')
+        display = ', '.join(displays)
+    else:
+        display = '...'
+
+    # Return display
+    if is_dict:
+        display = '{' + display + '}'
+    elif isinstance(value, list):
+        display = '[' + display + ']'
+    else:
+        display = '(' + display + ')'
+
+    return display
+
+
+def value_to_display(value, minmax=False, level=0):
     """Convert value for display purpose"""
+    # To save current Numpy threshold
+    np_threshold = FakeObject
+
     try:
         numeric_numpy_types = (int64, int32, float64, float32,
                                complex128, complex64)
+        if ndarray is not FakeObject:
+            # Save threshold
+            np_threshold = get_printoptions().get('threshold')
+            # Set max number of elements to show for Numpy arrays
+            # in our display
+            set_printoptions(threshold=10)
         if isinstance(value, recarray):
-            fields = value.names
-            display = 'Field names: ' + ', '.join(fields)
+            if level == 0:
+                fields = value.names
+                display = 'Field names: ' + ', '.join(fields)
+            else:
+                display = 'Recarray'
         elif isinstance(value, MaskedArray):
             display = 'Masked array'
         elif isinstance(value, ndarray):
-            if minmax:
-                try:
-                    display = 'Min: %r\nMax: %r' % (value.min(), value.max())
-                except (TypeError, ValueError):
+            if level == 0:
+                if minmax:
+                    try:
+                        display = 'Min: %r\nMax: %r' % (value.min(), value.max())
+                    except (TypeError, ValueError):
+                        if value.dtype.type in numeric_numpy_types:
+                            display = repr(value)
+                        else:
+                            display = default_display(value)
+                elif value.dtype.type in numeric_numpy_types:
                     display = repr(value)
+                else:
+                    display = default_display(value)
             else:
-                display = repr(value)
-        elif isinstance(value, (list, tuple, dict, set)):
-            display = CollectionsRepr.repr(value)
+                display = 'Numpy array'
+        elif any([type(value) == t for t in [list, tuple, dict]]):
+            display = collections_display(value, level+1)
         elif isinstance(value, Image):
-            display = '%s  Mode: %s' % (address(value), value.mode)
-        elif isinstance(value, DataFrame):
-            cols = value.columns
-            if PY2 and len(cols) > 0:
-                # Get rid of possible BOM utf-8 data present at the
-                # beginning of a file, which gets attached to the first
-                # column header when headers are present in the first
-                # row.
-                # Fixes Issue 2514
-                try:
-                    ini_col = to_text_string(cols[0], encoding='utf-8-sig')
-                except:
-                    ini_col = to_text_string(cols[0])
-                cols = [ini_col] + [to_text_string(c) for c in cols[1:]]
+            if level == 0:
+                display = '%s  Mode: %s' % (address(value), value.mode)
             else:
-                cols = [to_text_string(c) for c in cols]
-            display = 'Column names: ' + ', '.join(list(cols))
+                display = 'Image'
+        elif isinstance(value, DataFrame):
+            if level == 0:
+                cols = value.columns
+                if PY2 and len(cols) > 0:
+                    # Get rid of possible BOM utf-8 data present at the
+                    # beginning of a file, which gets attached to the first
+                    # column header when headers are present in the first
+                    # row.
+                    # Fixes Issue 2514
+                    try:
+                        ini_col = to_text_string(cols[0], encoding='utf-8-sig')
+                    except:
+                        ini_col = to_text_string(cols[0])
+                    cols = [ini_col] + [to_text_string(c) for c in cols[1:]]
+                else:
+                    cols = [to_text_string(c) for c in cols]
+                display = 'Column names: ' + ', '.join(list(cols))
+            else:
+                display = 'Dataframe'
         elif isinstance(value, NavigableString):
             # Fixes Issue 2448
             display = to_text_string(value)
+            if level > 0:
+                display = u"'" + display + u"'"
         elif isinstance(value, DatetimeIndex):
-            display = value.summary()
+            if level == 0:
+                display = value.summary()
+            else:
+                display = 'DatetimeIndex'
         elif is_binary_string(value):
             try:
                 display = to_text_string(value, 'utf8')
             except:
                 display = value
+            if level > 0:
+                display = (to_binary_string("'") + display +
+                           to_binary_string("'"))
         elif is_text_string(value):
             display = value
-        elif isinstance(value, NUMERIC_TYPES) or isinstance(value, bool) or \
-          isinstance(value, datetime.date) or \
-          isinstance(value, numeric_numpy_types):
+            if level > 0:
+                display = u"'" + display + u"'"
+        elif (isinstance(value, NUMERIC_TYPES) or isinstance(value, bool) or
+              isinstance(value, datetime.date) or
+              isinstance(value, datetime.timedelta) or
+              isinstance(value, numeric_numpy_types)):
             display = repr(value)
         else:
-            # Note: Don't trust on repr's. They can be inefficient and
-            # so freeze Spyder quite easily
-            # display = repr(value)
-            type_str = to_text_string(type(value))
-            display = type_str[1:-1]
+            if level == 0:
+                display = default_display(value)
+            else:
+                display = default_display(value, with_module=False)
     except:
-        type_str = to_text_string(type(value))
-        display = type_str[1:-1]
+        display = default_display(value)
 
-    # Truncate display at 80 chars to avoid freezing Spyder
+    # Truncate display at 70 chars to avoid freezing Spyder
     # because of large displays
-    if len(display) > 80:
-        display = display[:80].rstrip() + ' ...'
+    if len(display) > 70:
+        display = display[:70].rstrip() + ' ...'
+
+    # Restore Numpy threshold
+    if np_threshold is not FakeObject:
+        set_printoptions(threshold=np_threshold)
 
     return display
 
 
-
 def display_to_value(value, default_value, ignore_errors=True):
     """Convert back to value"""
     from qtpy.compat import from_qvariant
@@ -365,6 +481,8 @@ def display_to_value(value, default_value, ignore_errors=True):
             value = datestr_to_datetime(value)
         elif isinstance(default_value, datetime.date):
             value = datestr_to_datetime(value).date()
+        elif isinstance(default_value, datetime.timedelta):
+            value = str_to_timedelta(value)
         elif ignore_errors:
             value = try_to_eval(value)
         else:
@@ -418,20 +536,24 @@ def get_human_readable_type(item):
 # Globals filter: filter namespace dictionaries (to be edited in
 # CollectionsEditor)
 #==============================================================================
-def is_supported(value, check_all=False, filters=None, iterate=True):
+def is_supported(value, check_all=False, filters=None, iterate=False):
     """Return True if the value is supported, False otherwise"""
     assert filters is not None
+    if value is None:
+        return True
     if not is_editable_type(value):
         return False
     elif not isinstance(value, filters):
         return False
     elif iterate:
         if isinstance(value, (list, tuple, set)):
+            valid_count = 0
             for val in value:
-                if not is_supported(val, filters=filters, iterate=check_all):
-                    return False
+                if is_supported(val, filters=filters, iterate=check_all):
+                    valid_count += 1
                 if not check_all:
                     break
+            return valid_count > 0
         elif isinstance(value, dict):
             for key, val in list(value.items()):
                 if not is_supported(key, filters=filters, iterate=check_all) \
@@ -468,7 +590,7 @@ def globalsfilter(input_dict, check_all=False, filters=None,
 #==============================================================================
 REMOTE_SETTINGS = ('check_all', 'exclude_private', 'exclude_uppercase',
                    'exclude_capitalized', 'exclude_unsupported',
-                   'excluded_names', 'minmax', 'autorefresh')
+                   'excluded_names', 'minmax')
 
 
 def get_remote_data(data, settings, mode, more_excluded_names=None):
